#!/usr/local/bin/perl -w
#
#
#  Author: Preslav Nakov
#          UC Berkeley
#          nakov@cs.berkeley.edu
#
#  Modified for TREC 2003:
#          Ravi Teja Bhupatiraju
#          OHSU
#          bhupatir@ohsu.edu
#  
#  Description: Demonstrates the use of the calc_dice module
#            1) Original Dice                : CalcDice_Classic ($text1, $text2, $explain?)
#            2) Modified unigram Dice        : CalcDice_Modified_Unigram ($text1, $text2, $explain?)
#            3) Modified bigram Dice         : CalcDice_Modified_Bigram ($text1, $text2, $explain?)
#            4) Modified bigram Dice phrases : CalcDice_Modified_Bigram_Phrases ($text1, $text2, $explain?)
#
#  Principle Modifications:
#            1.) changes to accomodate input format changes
#            2.) aggregate statistics
#
#  Last modified: July 10, 2003
#
#

#use strict;
#-----------------------------------------------------------------------------------------------------------------

############################
## Change format for dice ##
############################

$argument_count = @ARGV;

if (not($argument_count == 2 or $argument_count == 3))
{
    die "Usage: calcDice.pl SecondaryTaskTextFile CandidateTextFile\n";
}

$sec_file = $ARGV[0];
$cand_file = $ARGV[1];

#start redirecting
open(STDOUT, ">report.txt") || die "can't open output: $!\nThe STDOUT is redirected to this file\n";


# check if the files exist
if (!(-e $sec_file && -e $cand_file))
{
    die "One or more input files do not exist\n";
}

#read sec
open FILE, $sec_file || die "Could not open Secondary Task Text file\n";
@sec = <FILE>;
close FILE;

#create sec array of text with split
foreach $line (@sec)
{
    @elements = split /\t/, $line;
    chomp $elements[1];
    push @sec_text, $elements[1];
}
$sec_length = @sec_text;


#read cand
open FILE, $cand_file || die "Could not open Candidate Text file\n";
@cand = <FILE>;
close FILE;

#create cand array of text with split
foreach $line (@cand)
{
    @elements = split /\t/, $line;
    chomp $elements[1];
    push @cand_text, $elements[1];
}
$cand_length = @cand_text;

#check if the lengths are OK!
$sec_length == $cand_length || die "Oops! The files to match have different lengths\n";

#Create Output
for($x=0; $x<$sec_length; $x++)
{
    $text1 = $sec_text[$x];
    $text2 = $cand_text[$x];
    
    push @output, "<TEXT1>$text1</TEXT1><TEXT2>$text2</TEXT2>\n";
}

#Write output to file.
open OUT, ">dice.txt" || die "Could not write to out.txt\n";  # dice format input
print OUT @output;
close OUT;

#-----------------------------------------------------------------------------------------------------------------

use calc_dice;

#############################
###   PROGRAM PARAMETERS  ###
#############################

if ($argument_count == 3 and $ARGV[2] eq "-v")
{
    $PRINT_INFO = 1;
}
else
{
    $PRINT_INFO = 0;
}


#######################
###   MAIN PROGRAM  ###
#######################
open(INPUT, "dice.txt") or die ('Failed to open $file for reading!\n');
$counter = 0;
while (<INPUT>)
{
	$counter++;
	my ($text1, $text2);
	
	### 1. Get the two texts
	die "Incorrect file format!" if (!/<TEXT1>(.*)<\/TEXT1>.*<TEXT2>(.*)<\/TEXT2>/);
	$text1 = $1;
	$text2 = $2;
	print "Set: $counter\n";
	print "*********************\n" if ($PRINT_INFO);
	print "*** ORIGINAL TEXT ***\n" if ($PRINT_INFO);
	print "*********************" if ($PRINT_INFO);
	print "\nTEXT1: $text1\nTEXT2: $text2\n";
	
	### 2. Output the classic Dice coefficient
	print "\n\n********************\n" if ($PRINT_INFO);
	print "*** CLASSIC DICE ***\n" if ($PRINT_INFO);
	print "********************" if ($PRINT_INFO);
	printf "\nClassic Dice:                 %0.2f%%\n", 100.0 * CalcDice_Classic($text1,$text2,$PRINT_INFO);
	
	print "\n\n*****************************\n" if ($PRINT_INFO);
	print "*** MODIFIED UNIGRAM DICE ***\n" if ($PRINT_INFO);
	print "*****************************\n" if ($PRINT_INFO);
	printf "Modified unigram Dice:        %0.2f%%\n", 100.0 * CalcDice_Modified_Unigram($text1,$text2,$PRINT_INFO);
	
	print "\n\n****************************\n" if ($PRINT_INFO);
	print "*** MODIFIED BIGRAM DICE ***\n" if ($PRINT_INFO);
	print "****************************\n" if ($PRINT_INFO);
	printf "Modified bigram Dice:         %0.2f%%\n", 100.0 * CalcDice_Modified_Bigram($text1,$text2,$PRINT_INFO);
	
	print "\n\n************************************\n" if ($PRINT_INFO);
	print "*** MODIFIED BIGRAM DICE PHRASES ***\n" if ($PRINT_INFO);
	print "************************************\n" if ($PRINT_INFO);
	printf "Modified bigram Dice phrases: %0.2f%%\n", 100.0 * CalcDice_Modified_Bigram_Phrases($text1,$text2,$PRINT_INFO);
	
	printf "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n\n";
	
}
close(INPUT);
unlink "dice.txt";
#-----------------------------------------------------------------------------------------------------------------

#############################################
##########   Aggregate Statistics   #########
#############################################

# read report (from STDOUT)
open FILE, 'report.txt' or die "Could not read temporary file <dice_results.txt>\n";
@contents = <FILE>;
close FILE;
# remove the temporary file.

# define patterns to parse the values
$pat_classic = "Classic Dice:[\ ]*?([0-9|\.]{4,6})";
$pat_unigram = "Modified unigram Dice:[\ ]*?([0-9|\.]{4,6})";
$pat_bigram = "Modified bigram Dice:[\ ]*?([0-9|\.]{4,6})";
$pat_modified_bigram = "Modified bigram Dice phrases:[\ ]*?([0-9|\.]{4,6})";

# load arrays with values
for $line (@contents)
{
    if ($line =~ /$pat_classic/)
    {
        push @classic, $1;
    }
    
    if ($line =~ /$pat_unigram/)
    {
        push @unigram, $1;
    }
    
    if ($line =~ /$pat_bigram/)
    {
        push @bigram, $1;
    }
    
    if ($line =~ /$pat_modified_bigram/)
    {
        push @modified_bigram, $1;
    }	
}

# get sizes
$classic = @classic;
$unigram = @unigram;
$bigram = @bigram;
$modified_bigram = @modified_bigram;

# check if all have equal sizes
# redundant.
if (!(($classic == $unigram) && ($classic == $modified_bigram) && ($classic == $modified_bigram)))
{
    die "The statistics have to be collected evenly";
}
#calc aggregates
for $value (@classic)
{
    $sum_classic += $value;
}

for $value (@unigram)
{
    $sum_unigram += $value;
}

for $value (@bigram)
{
    $sum_bigram += $value;
}

for $value (@modified_bigram)
{
    $sum_modified_bigram += $value;
}

$avg_classic = sprintf ("%1.2f", $sum_classic/$classic);
$avg_unigram = sprintf ("%1.2f", $sum_unigram/$unigram);
$avg_bigram = sprintf ("%1.2f", $sum_bigram/$bigram);
$avg_modified_bigram = sprintf ("%1.2f", $sum_modified_bigram/$modified_bigram);

# write to file
open FILE, ">>report.txt";
print FILE "\n\n\n";
print FILE "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n";
print FILE "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<  Average  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n";
print FILE "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n";
print FILE "Classic Dice:                 $avg_classic%\n";
print FILE "Modified unigram Dice:        $avg_unigram%\n";
print FILE "Modified bigram Dice:         $avg_bigram%\n";
print FILE "Modified bigram Dice phrases: $avg_modified_bigram%\n";
print FILE "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n";
close FILE;

#-----------------------------------------------------------------------------------------------------------------

# stop redirecting
close (STDOUT) || die "failed to close redirection from STDOUT";